@@ -3,9 +3,9 @@
 # see extra/config/Kconfig-language.txt
 #
 
-mainmenu "uClibc C Library Configuration"
-
-menu "Target Architecture Features and Options"
+config TARGET_ARCH
+	default "sh" if CONFIG_SH2 || CONFIG_SH3 || CONFIG_SH4
+	default "sh64" if CONFIG_SH5
 
 config HAVE_ELF
 	bool
@@ -38,53 +38,30 @@ choice
 	  that uClibc will work at all....
 
 	  Here are the available choices:
-	  - "SH2" Hitachi SH2
-	  - "SH3" Hitachi SH3
-	  - "SH4" Hitachi SH4
-	  - "SH5" Hitachi SH5
+	  - "SH2" SuperH SH-2
+	  - "SH3" SuperH SH-3
+	  - "SH4" SuperH SH-4
+	  - "SH5" SuperH SH-5 101, 103
 
 config CONFIG_SH2
+	select ARCH_HAS_NO_MMU
+	select ARCH_HAS_NO_LDSO
 	bool "SH2"
 
 config CONFIG_SH3
+	select UCLIBC_HAS_MMU
 	bool "SH3"
 
 config CONFIG_SH4
+	select UCLIBC_HAS_MMU
+	select FORCE_SHAREABLE_TEXT_SEGMENTS
 	bool "SH4"
 
 config CONFIG_SH5
+	select UCLIBC_HAS_MMU
+	select ARCH_HAS_NO_LDSO
 	bool "SH5"
 
 endchoice
 
-choice
-	prompt "Target Processor Endianness"
-	default ARCH_LITTLE_ENDIAN
-	help
-	  This is the endianness you wish to build use.  Choose either Big
-	  Endian, or Little Endian.
-
-config ARCH_LITTLE_ENDIAN
-	bool "Little Endian"
-
-config ARCH_BIG_ENDIAN
-	bool "Big Endian"
-
-endchoice
-
-
-config ARCH_HAS_NO_MMU
-	bool
-	default y if CONFIG_SH2
-
-config ARCH_HAS_NO_FPU
-       bool
-       default y if CONFIG_SH2 || CONFIG_SH3
-
-source "extra/Configs/Config.in.arch"
-
-endmenu
-
-source "extra/Configs/Config.in"
-
 
