@@ -246,8 +246,10 @@
 #define __STDC_IEC_559__		1
 #define __STDC_IEC_559_COMPLEX__	1
 
+#ifdef __UCLIBC_HAS_WCHAR__
 /* wchar_t uses ISO 10646-1 (2nd ed., published 2000-09-15) / Unicode 3.0.  */
 #define __STDC_ISO_10646__		200009L
+#endif
 
 /* This macro indicates that the installed library is uClibc.  Use
  * __UCLIBC_MAJOR__ and __UCLIBC_MINOR__ to test for the features in
@@ -337,7 +339,7 @@
 #undef __need_uClibc_config_h
 
 /* Make sure users large file options agree with uClibc's configuration. */
-#ifndef __UCLIBC_HAVE_LFS__
+#ifndef __UCLIBC_HAS_LFS__
 
 /* If uClibc was built without large file support, output an error if
  * and 64-bit file offsets were requested, output an error.
@@ -371,7 +373,7 @@ uClibc was built without large file support enabled.
 
 /* Some nice features only work properly with ELF */
 #if defined _LIBC 
-#if defined HAVE_ELF	
+#if defined __HAVE_ELF__
 /* Define ALIASNAME as a weak alias for NAME. */
 #  define weak_alias(name, aliasname) _weak_alias (name, aliasname)
 #  define _weak_alias(name, aliasname) \
@@ -386,16 +388,20 @@ uClibc was built without large file support enabled.
 # define weak_const_function __attribute__ ((weak, __const__))
 /* Tacking on "\n\t#" to the section name makes gcc put it's bogus
  * section attributes on what looks like a comment to the assembler. */
-#  define link_warning(symbol, msg)					      \
+#  if defined(__cris__) 
+#    define link_warning(symbol, msg)
+#  else
+#    define link_warning(symbol, msg)					      \
 	asm (".section "  ".gnu.warning." #symbol  "\n\t.previous");	      \
 	    static const char __evoke_link_warning_##symbol[]		      \
-	    __attribute__ ((section (".gnu.warning." #symbol "\n\t#"))) = msg;
-#else /* !defined HAVE_ELF */
+	    __attribute__ ((unused, section (".gnu.warning." #symbol "\n\t#"))) = msg;
+#endif
+#else /* !defined __HAVE_ELF__ */
 #  define strong_alias(name, aliasname) _strong_alias (name, aliasname)
 #  define weak_alias(name, aliasname) _strong_alias (name, aliasname)
 #  define _strong_alias(name, aliasname) \
-	__asm__(".global " C_SYMBOL_PREFIX #aliasname "\n" \
-                ".set " C_SYMBOL_PREFIX #aliasname "," C_SYMBOL_PREFIX #name);
+	__asm__(".global " __C_SYMBOL_PREFIX__ #aliasname "\n" \
+                ".set " __C_SYMBOL_PREFIX__ #aliasname "," __C_SYMBOL_PREFIX__ #name);
 #  define link_warning(symbol, msg) \
 	asm (".stabs \"" msg "\",30,0,0,0\n\t" \
 	      ".stabs \"" #symbol "\",1,0,0,0\n");
@@ -421,14 +427,23 @@ uClibc was built without large file support enabled.
 #if __GNUC__ == 2 && __GNUC_MINOR__ < 96
 #define __builtin_expect(x, expected_value) (x)
 #endif
-
-#define likely(x)       __builtin_expect((x),1)
-#define unlikely(x)     __builtin_expect((x),0)
+#ifndef likely
+# define likely(x)	__builtin_expect((!!(x)),1)
+#endif
+#ifndef unlikely
+# define unlikely(x)	__builtin_expect((!!(x)),0)
+#endif
+#ifndef __LINUX_COMPILER_H
+#define __LINUX_COMPILER_H
+#endif
 
 /* --- this is added to integrate linuxthreads */
 #define __USE_UNIX98            1
 
 #endif /* _LIBC only stuff */
 
+#ifndef __linux__
+# define __linux__ 1
+#endif
 
 #endif	/* features.h  */
