@@ -2,8 +2,7 @@
 /*
  * Syscalls for uClibc
  *
- * Copyright (C) 2000 by Lineo, inc
- * Copyright (C) 2001, 2002 by Erik Andersen
+ * Copyright (C) 2001-2003 by Erik Andersen
  * Written by Erik Andersen <andersen@codpoet.org>
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -47,14 +46,14 @@
 #	ifdef __UCLIBC_HAS_MMU__
 #define __NR___libc_fork __NR_fork
 		_syscall0(pid_t, __libc_fork);
-		weak_alias (__libc_fork, fork)
 #	else
-		pid_t fork(void)
+		pid_t __libc_fork(void)
 		{
 			__set_errno(ENOSYS);
 			return -1;
 		}
 #	endif
+weak_alias (__libc_fork, fork)
 #endif
 
 //#define __NR_read             3
@@ -77,12 +76,40 @@ weak_alias (__libc_write, __write)
 #endif
 
 //#define __NR_open             5
-#ifdef L___libc_open
+#ifdef L___syscall_open
+#define __NR___syscall_open __NR_open
+#include <stdlib.h>
 #include <stdarg.h>
-/* Do not include fcntl.h, so gcc doesn't whine the prototype */
-#define __NR___libc_open __NR_open
-_syscall3(int, __libc_open, const char *, fn, int, flags, mode_t, mode);
+#include <fcntl.h>
+#include <string.h>
+#include <sys/param.h>
+static inline
+_syscall3(int, __syscall_open, const char *, file, int, flags, __kernel_mode_t, mode);
+int __libc_open (const char * file, int flags, ...)
+{
+	mode_t mode;
+	if (unlikely(file==NULL || *file=='\0')) {
+		__set_errno(EINVAL);
+		return -1;
+	}
+	if (unlikely(((strlen(file)+1) > NAME_MAX))) {
+		__set_errno(ENAMETOOLONG);
+		return -1;
+	}
+	if (flags & O_CREAT) {
+		va_list ap;
+		va_start(ap, flags);
+		mode = va_arg(ap, mode_t);
+		va_end(ap);
+	}
+	return __syscall_open(file, flags, mode);
+}
 weak_alias(__libc_open, open)
+
+int creat(const char *file, mode_t mode)
+{
+	  return __libc_open (file, O_WRONLY|O_CREAT|O_TRUNC, mode);
+}
 #endif
 
 //#define __NR_close            6
@@ -97,18 +124,7 @@ weak_alias(__libc_close, close)
 // Implemented using wait4 
 
 //#define __NR_creat            8
-#ifdef L_creat
-#include <fcntl.h>
-#ifdef __NR_creat
-_syscall2(int, creat, const char *, file, mode_t, mode);
-#else
-extern int __libc_open (const char *file, int flags, mode_t mode);
-int creat (const char *file, mode_t mode)
-{
-	  return __libc_open (file, O_WRONLY|O_CREAT|O_TRUNC, mode);
-}
-#endif
-#endif
+// Implemented using open
 
 //#define __NR_link             9
 #ifdef L_link
@@ -123,16 +139,47 @@ _syscall1(int, unlink, const char *, pathname);
 #endif
 
 //#define __NR_execve           11
-#ifdef L_execve
+#ifdef L___syscall_execve
+#define __NR___syscall_execve __NR_execve
 #include <unistd.h>
-_syscall3(int, execve, const char *, filename, char *const *, argv,
-		  char *const *, envp);
+#include <string.h>
+#include <sys/param.h>
+static inline
+_syscall3(int, __syscall_execve, const char *, filename, 
+		char *const *, argv, char *const *, envp);
+int execve(const char *filename, char *const *argv, char *const *envp)
+{ 
+	if (unlikely(filename==NULL || *filename=='\0')) {
+		__set_errno(EINVAL);
+		return -1;
+	}
+	if (unlikely(((strlen(filename)+1) > NAME_MAX))) {
+		__set_errno(ENAMETOOLONG);
+		return -1;
+	}
+	return __syscall_execve(filename, argv, envp);
+}
 #endif
 
 //#define __NR_chdir            12
-#ifdef L_chdir
-#include <unistd.h>
-_syscall1(int, chdir, const char *, path);
+#ifdef L___syscall_chdir
+#define __NR___syscall_chdir __NR_chdir
+#include <string.h>
+#include <sys/param.h>
+static inline
+_syscall1(int, __syscall_chdir, const char *, path);
+int chdir(const char *path)
+{ 
+	if (unlikely(path==NULL || *path=='\0')) {
+		__set_errno(EINVAL);
+		return -1;
+	}
+	if (unlikely(((strlen(path)+1) > NAME_MAX))) {
+		__set_errno(ENAMETOOLONG);
+		return -1;
+	}
+	return __syscall_chdir(path);
+}
 #endif
 
 //#define __NR_time             13
@@ -162,17 +209,23 @@ time_t time (time_t *t)
 _syscall3(int, __syscall_mknod, const char *, path, __kernel_mode_t, mode, __kernel_dev_t, dev);
 int mknod(const char *path, mode_t mode, dev_t dev)
 { 
-	__kernel_dev_t k_dev;
 	/* We must convert the dev_t value to a __kernel_dev_t */
+	__kernel_dev_t k_dev;
 	k_dev = ((major(dev) & 0xff) << 8) | (minor(dev) & 0xff);
 	return __syscall_mknod(path, mode, k_dev);
 }
 #endif
 
 //#define __NR_chmod            15
-#ifdef L_chmod
+#ifdef L___syscall_chmod
 #include <sys/stat.h>
-_syscall2(int, chmod, const char *, path, mode_t, mode);
+#define __NR___syscall_chmod __NR_chmod
+static inline 
+_syscall2(int, __syscall_chmod, const char *, path, __kernel_mode_t, mode);
+int chmod(const char *path, mode_t mode)
+{
+	return __syscall_chmod(path, mode);
+}
 #endif
 
 /* Old kernels don't have lchown -- do chown instead.  This
@@ -183,9 +236,21 @@ _syscall2(int, chmod, const char *, path, mode_t, mode);
 #endif
 
 //#define __NR_lchown           16
-#ifdef L_lchown
+#ifdef L___syscall_lchown
 #include <unistd.h>
-_syscall3(int, lchown, const char *, path, uid_t, owner, gid_t, group);
+#define __NR___syscall_lchown __NR_lchown
+static inline 
+_syscall3(int, __syscall_lchown, const char *, path, __kernel_uid_t, owner, __kernel_gid_t, group);
+int lchown(const char *path, uid_t owner, gid_t group)
+{
+	if (((owner + 1) > (uid_t) ((__kernel_uid_t) -1U))
+			|| ((group + 1) > (gid_t) ((__kernel_gid_t) -1U)))
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
+	return __syscall_lchown(path, owner, group);
+}
 #endif
 
 
@@ -228,22 +293,41 @@ _syscall1(int, umount, const char *, specialfile);
 #endif
 
 //#define __NR_setuid           23
-// See setuid.c
+#ifdef L___syscall_setuid
+#define __NR___syscall_setuid __NR_setuid
+#include <unistd.h>
+static inline 
+_syscall1(int, __syscall_setuid, __kernel_uid_t, uid);
+int setuid(uid_t uid)
+{
+	if (uid == (uid_t) ~0 || uid != (uid_t) ((__kernel_uid_t) uid)) {
+		__set_errno (EINVAL);
+		return -1;
+	}
+	return(__syscall_setuid(uid));
+}
+#endif
 
 //#define __NR_getuid           24
-#ifdef L_getuid
+#ifdef L___syscall_getuid
 #include <unistd.h>
 #if defined (__alpha__)
 #define __NR_getuid     __NR_getxuid
 #endif
-_syscall0(uid_t, getuid);
+#define __NR___syscall_getuid __NR_getuid
+static inline 
+_syscall0(int, __syscall_getuid);
+uid_t getuid(void)
+{
+	return(__syscall_getuid());
+}
 #endif
 
 //#define __NR_stime            25
 #ifdef L_stime
 #include <time.h>
 #include <sys/time.h>
-#ifdef _NR_stime
+#ifdef __NR_stime
 _syscall1(int, stime, const time_t *, t);
 #else
 int stime(const time_t *when)
@@ -258,12 +342,7 @@ int stime(const time_t *when)
 #endif
 
 //#define __NR_ptrace           26
-#ifdef L___ptrace
-#include <sys/ptrace.h>
-#define __NR___ptrace __NR_ptrace
-_syscall4(long, __ptrace, enum __ptrace_request, request, pid_t, pid,
-		void*, addr, void*, data);
-#endif
+//See ptrace.c
 
 
 //#define __NR_alarm            27
@@ -409,22 +488,55 @@ int nice (int incr)
 //See sync.c
 
 //#define __NR_kill             37
-#ifdef L_kill
+#ifdef L___syscall_kill
 #include <signal.h>
 #undef kill
-_syscall2(int, kill, pid_t, pid, int, sig);
+#define __NR___syscall_kill __NR_kill
+static inline
+_syscall2(int, __syscall_kill, __kernel_pid_t, pid, int, sig);
+int kill(pid_t pid, int sig)
+{
+	return(__syscall_kill(pid, sig));
+}
 #endif
 
 //#define __NR_rename           38
-#ifdef L_rename
+#ifdef L___syscall_rename
+#define __NR___syscall_rename __NR_rename
+#include <unistd.h>
+#include <string.h>
+#include <sys/param.h>
 #include <stdio.h>
-_syscall2(int, rename, const char *, oldpath, const char *, newpath);
+static inline
+_syscall2(int, __syscall_rename, const char *, oldpath, const char *, newpath);
+int rename(const char *oldpath, const char *newpath)
+{ 
+	if (unlikely((oldpath==NULL || newpath==NULL ||
+					*oldpath=='\0' || *newpath=='\0'))) 
+	{
+		__set_errno(EINVAL);
+		return -1;
+	}
+	if (unlikely(((strlen(oldpath)+1) >= NAME_MAX) || 
+				((strlen(newpath)+1) >= NAME_MAX))) 
+	{
+		__set_errno(ENAMETOOLONG);
+		return -1;
+	}
+	return __syscall_rename(oldpath, newpath);
+}
 #endif
 
 //#define __NR_mkdir            39
-#ifdef L_mkdir
+#ifdef L___syscall_mkdir
 #include <sys/stat.h>
-_syscall2(int, mkdir, const char *, pathname, mode_t, mode);
+#define __NR___syscall_mkdir __NR_mkdir
+static inline 
+_syscall2(int, __syscall_mkdir, const char *, pathname, __kernel_mode_t, mode);
+int mkdir(const char * pathname, mode_t mode)
+{
+	return(__syscall_mkdir(pathname, mode));
+}
 #endif
 
 //#define __NR_rmdir            40
@@ -456,27 +568,51 @@ _syscall1(clock_t, times, struct tms *, buf);
 //#define __NR_brk              45
 
 //#define __NR_setgid           46
-#ifdef L_setgid
+#ifdef L___syscall_setgid
 #include <unistd.h>
-_syscall1(int, setgid, gid_t, gid);
+#define __NR___syscall_setgid __NR_setgid
+static inline 
+_syscall1(int, __syscall_setgid, __kernel_gid_t, gid);
+int setgid(gid_t gid)
+{
+	if (gid == (gid_t) ~0
+			|| gid != (gid_t) ((__kernel_gid_t) gid))
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
+	return(__syscall_setgid(gid));
+}
 #endif
 
 //#define __NR_getgid           47
-#ifdef L_getgid
+#ifdef L___syscall_getgid
 #include <unistd.h>
+#define __NR___syscall_getgid __NR_getgid
 #if defined (__alpha__)
 #define __NR_getgid     __NR_getxgid
 #endif
-_syscall0(gid_t, getgid);
+static inline 
+_syscall0(int, __syscall_getgid);
+gid_t getgid(void)
+{
+	return(__syscall_getgid());
+}
 #endif
 
 //#define __NR_signal           48
 
 //#define __NR_geteuid          49
-#ifdef	L_geteuid
+#ifdef	L___syscall_geteuid
 #include <unistd.h>
 #	ifdef	__NR_geteuid
-	_syscall0(uid_t, geteuid);
+#define __NR___syscall_geteuid __NR_geteuid
+	static inline 
+	_syscall0(int, __syscall_geteuid);
+	uid_t geteuid(void)
+	{
+		return(__syscall_geteuid());
+	}
 #	else
 	uid_t geteuid(void)
 	{
@@ -486,10 +622,16 @@ _syscall0(gid_t, getgid);
 #endif
 
 //#define __NR_getegid          50
-#ifdef	L_getegid
+#ifdef	L___syscall_getegid
 #include <unistd.h>
 #	ifdef	__NR_getegid
-	_syscall0(gid_t, getegid);
+#define __NR___syscall_getegid __NR_getegid
+static inline 
+_syscall0(int, __syscall_getegid);
+gid_t getegid(void)
+{
+	return(__syscall_getegid());
+}
 #	else
 	gid_t getegid(void)
 	{
@@ -521,75 +663,115 @@ _syscall1(int, acct, const char *, filename);
 //#define __NR_lock             53
 
 //#define __NR_ioctl            54
-#ifdef L__ioctl
+#ifdef L___syscall_ioctl
 #include <stdarg.h>
 #include <sys/ioctl.h>
-#define __NR__ioctl __NR_ioctl
-extern int _ioctl(int fd, int request, void *arg);
-
-_syscall3(int, _ioctl, int, fd, int, request, void *, arg);
-
-int ioctl(int fd, unsigned long int request, ...)
-{
-	void *arg;
-	va_list list;
-
-	va_start(list, request);
-	arg = va_arg(list, void *);
-
-	va_end(list);
-	return _ioctl(fd, request, arg);
-}
+#define __NR___syscall_ioctl __NR_ioctl
+extern int __syscall_ioctl(int fd, int request, void *arg);
+_syscall3(int, __syscall_ioctl, int, fd, int, request, void *, arg);
+#if !defined (__powerpc__)
+#include "ioctl.c"
+/* Also see ioctl.c and powerpc/ioctl.c */
+#endif
 #endif
 
 //#define __NR_fcntl            55
-#ifdef L__fcntl
+#ifdef L___syscall_fcntl
 #include <stdarg.h>
 #include <fcntl.h>
-#define __NR__fcntl __NR_fcntl
-extern int _fcntl(int fd, int cmd, long arg);
-
-_syscall3(int, _fcntl, int, fd, int, cmd, long, arg);
-
-int __libc_fcntl(int fd, int command, ...)
+#define __NR___syscall_fcntl __NR_fcntl
+#ifdef __UCLIBC_HAS_LFS__
+static inline
+#endif
+_syscall3(int, __syscall_fcntl, int, fd, int, cmd, long, arg);
+int __libc_fcntl(int fd, int cmd, ...)
 {
 	long arg;
 	va_list list;
-
-	va_start(list, command);
+	if (cmd == F_GETLK64 || cmd == F_SETLK64 || cmd == F_SETLKW64) 
+	{
+			__set_errno(ENOSYS);
+			return -1;
+	}
+	va_start(list, cmd);
 	arg = va_arg(list, long);
-
 	va_end(list);
-	return _fcntl(fd, command, arg);
+	return(__syscall_fcntl(fd, cmd, arg));
 }
 weak_alias(__libc_fcntl, fcntl)
+#if ! defined __NR_fcntl64 && defined __UCLIBC_HAS_LFS__
+weak_alias(__libc_fcntl, fcntl64);
+#endif
 #endif
 
 //#define __NR_mpx              56
 
 //#define __NR_setpgid          57
-#ifdef L_setpgid
+#ifdef L___syscall_setpgid
 #include <unistd.h>
-_syscall2(int, setpgid, pid_t, pid, pid_t, pgid);
+#define __NR___syscall_setpgid __NR_setpgid
+static inline
+_syscall2(int, __syscall_setpgid, __kernel_pid_t, pid, __kernel_pid_t, pgid);
+int setpgid(pid_t pid, pid_t pgid)
+{
+	return(__syscall_setpgid(pid, pgid));
+}
 #endif
 
 //#define __NR_ulimit           58
+//See ulimit.c
 
 //#define __NR_oldolduname      59
 
 //#define __NR_umask            60
-#ifdef L_umask
+#ifdef L___syscall_umask
 #include <sys/stat.h>
-_syscall1(mode_t, umask, mode_t, mask);
+#define __NR___syscall_umask __NR_umask
+static inline 
+_syscall1(__kernel_mode_t, __syscall_umask, __kernel_mode_t, mode);
+mode_t umask(mode_t mode)
+{
+	return(__syscall_umask(mode));
+}
 #endif
 
 //#define __NR_chroot           61
-#ifdef L_chroot
+#ifdef L___syscall_chroot
+#define __NR___syscall_chroot __NR_chroot
 #include <unistd.h>
-_syscall1(int, chroot, const char *, path);
+#include <string.h>
+#include <sys/param.h>
+static inline
+_syscall1(int, __syscall_chroot, const char *, path);
+int chroot(const char *path)
+{ 
+	if (unlikely(path==NULL || *path=='\0')) {
+		__set_errno(EINVAL);
+		return -1;
+	}
+	if (unlikely(((strlen(path)+1) > NAME_MAX))) {
+		__set_errno(ENAMETOOLONG);
+		return -1;
+	}
+	return __syscall_chroot(path);
+}
 #endif
 
 //#define __NR_ustat            62
+#ifdef L___syscall_ustat
+#define __NR___syscall_ustat __NR_ustat
+#include <sys/ustat.h>
+static inline
+_syscall2(int, __syscall_ustat, unsigned short int, kdev_t, struct ustat *, ubuf);
+int ustat(dev_t dev, struct ustat *ubuf)
+{ 
+	/* We must convert the dev_t value to a __kernel_dev_t */
+	__kernel_dev_t k_dev;
+	k_dev = ((major(dev) & 0xff) << 8) | (minor(dev) & 0xff);
+	return __syscall_ustat(k_dev, ubuf);
+}
+#endif
+
 
 //#define __NR_dup2             63
 #ifdef L_dup2
@@ -624,8 +806,8 @@ _syscall0(pid_t, setsid);
 
 //#define __NR_sigaction        67
 #ifndef __NR_rt_sigaction
-#define __NR___syscall_sigaction __NR_sigaction
 #ifdef L___syscall_sigaction
+#define __NR___syscall_sigaction __NR_sigaction
 #include <signal.h>
 #undef sigaction
 _syscall3(int, __syscall_sigaction, int, signum, const struct sigaction *, act,
@@ -638,15 +820,39 @@ _syscall3(int, __syscall_sigaction, int, signum, const struct sigaction *, act,
 //#define __NR_ssetmask         69
 
 //#define __NR_setreuid         70
-#ifdef L_setreuid
+#ifdef L___syscall_setreuid
 #include <unistd.h>
-_syscall2(int, setreuid, uid_t, ruid, uid_t, euid);
+#define __NR___syscall_setreuid __NR_setreuid
+static inline
+_syscall2(int, __syscall_setreuid, __kernel_uid_t, ruid, __kernel_uid_t, euid);
+int setreuid(uid_t ruid, uid_t euid)
+{
+	if (((ruid + 1) > (uid_t) ((__kernel_uid_t) -1U))
+			|| ((euid + 1) > (uid_t) ((__kernel_uid_t) -1U)))
+	{
+		__set_errno(EINVAL);
+		return -1;
+	}
+	return(__syscall_setreuid(ruid, euid));
+}
 #endif
 
 //#define __NR_setregid         71
-#ifdef L_setregid
+#ifdef L___syscall_setregid
 #include <unistd.h>
-_syscall2(int, setregid, gid_t, rgid, gid_t, egid);
+#define __NR___syscall_setregid __NR_setregid
+static inline
+_syscall2(int, __syscall_setregid, __kernel_gid_t, rgid, __kernel_gid_t, egid);
+int setregid(gid_t rgid, gid_t egid)
+{
+	if (((rgid + 1) > (gid_t) ((__kernel_gid_t) -1U))
+			|| ((egid + 1) > (gid_t) ((__kernel_gid_t) -1U)))
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
+	return(__syscall_setregid(rgid, egid));
+}
 #endif
 
 //#define __NR_sigsuspend       72
@@ -757,16 +963,56 @@ _syscall2(int, settimeofday, const struct timeval *, tv,
 #endif
 
 //#define __NR_getgroups        80
-#ifdef L_getgroups
+#ifdef L___syscall_getgroups
 #include <unistd.h>
-_syscall2(int, getgroups, int, size, gid_t *, list);
+#define __NR___syscall_getgroups __NR_getgroups
+static inline
+_syscall2(int, __syscall_getgroups, int, size, __kernel_gid_t *, list);
+#define MIN(a,b) (((a)<(b))?(a):(b))
+int getgroups(int n, gid_t *groups)
+{
+	if (unlikely(n < 0)) {
+		__set_errno(EINVAL);
+		return -1;
+	} else {
+		int i, ngids;
+		__kernel_gid_t kernel_groups[n = MIN(n, sysconf(_SC_NGROUPS_MAX))];
+		ngids = __syscall_getgroups(n, kernel_groups);
+		if (n != 0 && ngids > 0) {
+			for (i = 0; i < ngids; i++) {
+				groups[i] = kernel_groups[i];
+			}
+		}
+		return ngids;
+	}
+}
 #endif
 
 //#define __NR_setgroups        81
-#ifdef L_setgroups
+#ifdef L___syscall_setgroups
 #include <unistd.h>
 #include <grp.h>
-_syscall2(int, setgroups, size_t, size, const gid_t *, list);
+#define __NR___syscall_setgroups __NR_setgroups
+static inline
+_syscall2(int, __syscall_setgroups, size_t, size, const __kernel_gid_t *, list);
+int setgroups (size_t n, const gid_t *groups)
+{
+	if (n > (size_t)sysconf(_SC_NGROUPS_MAX)) {
+		__set_errno (EINVAL);
+		return -1;
+	} else {
+		size_t i;
+		__kernel_gid_t kernel_groups[n];
+		for (i = 0; i < n; i++) {
+			kernel_groups[i] = (groups)[i];
+			if (groups[i] != (gid_t) ((__kernel_gid_t)groups[i])) {
+				__set_errno (EINVAL);
+				return -1;
+			}
+		}
+		return(__syscall_setgroups(n, kernel_groups));
+	}
+}
 #endif
 
 //#define __NR_select           82
@@ -866,15 +1112,33 @@ _syscall2(int, ftruncate, int, fd, __off_t, length);
 #endif
 
 //#define __NR_fchmod           94
-#ifdef L_fchmod
+#ifdef L___syscall_fchmod
 #include <sys/stat.h>
-_syscall2(int, fchmod, int, fildes, mode_t, mode);
+#define __NR___syscall_fchmod __NR_fchmod
+static inline 
+_syscall2(int, __syscall_fchmod, int, fildes, __kernel_mode_t, mode);
+int fchmod(int fildes, mode_t mode)
+{
+	return(__syscall_fchmod(fildes, mode));
+}
 #endif
 
 //#define __NR_fchown           95
-#ifdef L_fchown
+#ifdef L___syscall_fchown
 #include <unistd.h>
-_syscall3(int, fchown, int, fd, uid_t, owner, gid_t, group);
+#define __NR___syscall_fchown __NR_fchown
+static inline
+_syscall3(int, __syscall_fchown, int, fd, __kernel_uid_t, owner, __kernel_gid_t, group);
+int fchown(int fd, uid_t owner, gid_t group)
+{
+	if (((owner + 1) > (uid_t) ((__kernel_uid_t) -1U))
+			|| ((group + 1) > (gid_t) ((__kernel_gid_t) -1U)))
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
+	return(__syscall_fchown(fd, owner, group));
+}
 #endif
 
 //#define __NR_getpriority      96
@@ -905,9 +1169,25 @@ _syscall3(int, setpriority, __priority_which_t, which, id_t, who, int, prio);
 //#define __NR_profil           98
 
 //#define __NR_statfs           99
-#ifdef L_statfs
+#ifdef L___syscall_statfs
+#define __NR___syscall_statfs __NR_statfs
+#include <string.h>
+#include <sys/param.h>
 #include <sys/vfs.h>
-_syscall2(int, statfs, const char *, path, struct statfs *, buf);
+static inline
+_syscall2(int, __syscall_statfs, const char *, path, struct statfs *, buf);
+int statfs(const char *path, struct statfs *buf)
+{ 
+	if (unlikely(path==NULL || *path=='\0')) {
+		__set_errno(EINVAL);
+		return -1;
+	}
+	if (unlikely(((strlen(path)+1) > NAME_MAX))) {
+		__set_errno(ENAMETOOLONG);
+		return -1;
+	}
+	return __syscall_statfs(path, buf);
+}
 #endif
 
 //#define __NR_fstatfs          100
@@ -969,7 +1249,8 @@ _syscall2(int, getitimer, __itimer_which_t, which, struct itimerval *, value);
 #ifdef L___syscall_stat
 #define __NR___syscall_stat __NR_stat
 #include <unistd.h>
-#include <sys/stat.h>
+#define _SYS_STAT_H
+#include <bits/stat.h>
 #include "xstatconv.h"
 _syscall2(int, __syscall_stat, const char *, file_name, struct kernel_stat *, buf);
 int stat(const char * file_name, struct stat * buf)
@@ -991,7 +1272,8 @@ weak_alias(stat, stat64);
 #ifdef L___syscall_lstat
 #define __NR___syscall_lstat __NR_lstat
 #include <unistd.h>
-#include <sys/stat.h>
+#define _SYS_STAT_H
+#include <bits/stat.h>
 #include <bits/kernel_stat.h>
 #include "xstatconv.h"
 _syscall2(int, __syscall_lstat, const char *, file_name, struct kernel_stat *, buf);
@@ -1014,7 +1296,8 @@ weak_alias(lstat, lstat64);
 #ifdef L___syscall_fstat
 #define __NR___syscall_fstat __NR_fstat
 #include <unistd.h>
-#include <sys/stat.h>
+#define _SYS_STAT_H
+#include <bits/stat.h>
 #include <bits/kernel_stat.h>
 #include "xstatconv.h"
 _syscall2(int, __syscall_fstat, int, fd, struct kernel_stat *, buf);
@@ -1064,8 +1347,14 @@ _syscall0(int, vhangup);
 //#define __NR_vm86old          113
 
 //#define __NR_wait4            114
-#ifdef L_wait4
-_syscall4(int, wait4, pid_t, pid, int *, status, int, opts, void *, rusage);
+#ifdef L___syscall_wait4
+#define __NR___syscall_wait4 __NR_wait4
+static inline
+_syscall4(int, __syscall_wait4, __kernel_pid_t, pid, int *, status, int, opts, void *, rusage);
+int wait4(pid_t pid, int * status, int opts, void * rusage)
+{
+	return(__syscall_wait4(pid, status, opts, rusage));
+}
 #endif
 
 //#define __NR_swapoff          115
@@ -1118,6 +1407,7 @@ _syscall1(int, uname, struct utsname *, buf);
 #ifdef __NR_modify_ldt
 #ifdef L_modify_ldt
 _syscall3(int, modify_ldt, int, func, void *, ptr, unsigned long, bytecount);
+weak_alias(modify_ldt, __modify_ldt);
 #endif
 #endif
 
@@ -1126,6 +1416,7 @@ _syscall3(int, modify_ldt, int, func, void *, ptr, unsigned long, bytecount);
 #include <sys/timex.h>
 _syscall1(int, adjtimex, struct timex *, buf);
 weak_alias(adjtimex, __adjtimex);
+weak_alias(adjtimex, ntp_adjtime);
 #endif
 
 //#define __NR_mprotect         125
@@ -1184,8 +1475,14 @@ _syscall4(int, quotactl, int, cmd, const char *, special , int, id, caddr_t, add
 #endif
 
 //#define __NR_getpgid          132
-#ifdef L_getpgid
-_syscall1(pid_t, getpgid, pid_t, pid);
+#ifdef L___syscall_getpgid
+#define __NR___syscall_getpgid __NR_getpgid
+static inline
+_syscall1(__kernel_pid_t, __syscall_getpgid, __kernel_pid_t, pid);
+pid_t getpgid(pid_t pid)
+{
+	return(__syscall_getpgid(pid));
+}
 #endif
 
 //#define __NR_fchdir           133
@@ -1201,24 +1498,54 @@ _syscall2(int, bdflush, int, __func, long int, __data);
 #endif
 
 //#define __NR_sysfs            135
+//_syscall3(int, sysfs, int, option, unsigned int, index, char addr);
+
 
 //#define __NR_personality      136
+#ifdef __NR_personality
+#ifdef L_personality
+#include <sys/personality.h>
+_syscall1(int, personality, unsigned long int, __persona);
+#endif
+#endif
 
 //#define __NR_afs_syscall      137
 
 //#define __NR_setfsuid         138
 #ifdef __NR_setfsuid
-#ifdef L_setfsuid
+#ifdef L___syscall_setfsuid
 #include <sys/fsuid.h>
-_syscall1(int, setfsuid, uid_t, uid);
+#define __NR___syscall_setfsuid __NR_setfsuid
+static inline
+_syscall1(int, __syscall_setfsuid, __kernel_uid_t, uid);
+int setfsuid(uid_t uid)
+{
+	if (uid != (uid_t) ((__kernel_uid_t) uid))
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
+	return(__syscall_setfsuid(uid));
+}
 #endif
 #endif
 
 //#define __NR_setfsgid         139
 #ifdef __NR_setfsgid
-#ifdef L_setfsgid
+#ifdef L___syscall_setfsgid
 #include <sys/fsuid.h>
-_syscall1(int, setfsgid, gid_t, gid);
+#define __NR___syscall_setfsgid __NR_setfsgid
+static inline
+_syscall1(int, __syscall_setfsgid, __kernel_gid_t, gid);
+int setfsgid(gid_t gid)
+{
+	if (gid != (gid_t) ((__kernel_gid_t) gid))
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
+	return(__syscall_setfsgid(gid));
+}
 #endif
 #endif
 
@@ -1226,11 +1553,7 @@ _syscall1(int, setfsgid, gid_t, gid);
 //See llseek.c
 
 //#define __NR_getdents         141
-#ifdef L_getdents
-#include <unistd.h>
-#include <dirent.h>
-_syscall3(int, getdents, int, fd, char *, dirp, size_t, count);
-#endif
+// See getdents.c
 
 //#define __NR__newselect       142
 #ifdef L__newselect
@@ -1246,9 +1569,15 @@ weak_alias(_newselect, select);
 #endif
 
 //#define __NR_flock            143
-#ifdef L_flock
+#ifdef L___syscall_flock
 #include <sys/file.h>
-_syscall2(int,flock,int,fd, int,operation);
+#define __NR___syscall_flock __NR_flock
+static inline
+_syscall2(int, __syscall_flock, int, fd, int, operation);
+int flock(int fd, int operation)
+{
+	return(__syscall_flock(fd, operation));
+}
 #endif
 
 //#define __NR_msync            144
@@ -1275,9 +1604,15 @@ _syscall3(ssize_t, writev, int, filedes, const struct iovec *, vector, int,
 #endif
 
 //#define __NR_getsid           147
-#ifdef L_getsid
+#ifdef L___syscall_getsid
 #include <unistd.h>
-_syscall1(pid_t, getsid, pid_t, pid);
+#define __NR___syscall_getsid __NR_getsid
+static inline
+_syscall1(__kernel_pid_t, __syscall_getsid, __kernel_pid_t, pid);
+pid_t getsid(pid_t pid)
+{
+	return(__syscall_getsid(pid));
+}
 #endif
 
 //#define __NR_fdatasync        148
@@ -1291,7 +1626,15 @@ _syscall1(int, fdatasync, int, fd);
 //#define __NR__sysctl          149
 #ifdef __NR__sysctl
 #ifdef L__sysctl
-#include <linux/sysctl.h>
+struct __sysctl_args {
+	int *name;
+	int nlen;
+	void *oldval;
+	size_t *oldlenp;
+	void *newval;
+	size_t newlen;
+	unsigned long __unused[4];
+};
 _syscall1(int, _sysctl, struct __sysctl_args *, args);
 int sysctl(int *name, int nlen, void *oldval, size_t *oldlenp,
 			void *newval, size_t newlen)
@@ -1345,33 +1688,57 @@ int sysctl(int *name, int nlen, void *oldval, size_t *oldlenp,
 
 //#define __NR_sched_setparam   154
 #ifdef __NR_sched_setparam
-#ifdef L_sched_setparam
+#ifdef L___syscall_sched_setparam
 #include <sched.h>
-_syscall2(int, sched_setparam, pid_t, pid, const struct sched_param *, p);
+#define __NR___syscall_sched_setparam __NR_sched_setparam
+static inline
+_syscall2(int, __syscall_sched_setparam, __kernel_pid_t, pid, const struct sched_param *, p);
+int sched_setparam(pid_t pid, const struct sched_param * p)
+{
+	return(__syscall_sched_setparam(pid, p));
+}
 #endif
 #endif
 
 //#define __NR_sched_getparam   155
 #ifdef __NR_sched_getparam
-#ifdef L_sched_getparam
+#ifdef L___syscall_sched_getparam
 #include <sched.h>
-_syscall2(int, sched_getparam, pid_t, pid, struct sched_param *, p);
+#define __NR___syscall_sched_getparam __NR_sched_getparam
+static inline
+_syscall2(int, __syscall_sched_getparam, __kernel_pid_t, pid, struct sched_param *, p);
+int sched_getparam(pid_t pid, struct sched_param * p)
+{
+	return(__syscall_sched_getparam(pid, p));
+}
 #endif
 #endif
 
 //#define __NR_sched_setscheduler       156
 #ifdef __NR_sched_setscheduler
-#ifdef L_sched_setscheduler
+#ifdef L___syscall_sched_setscheduler
 #include <sched.h>
-_syscall3(int, sched_setscheduler, pid_t, pid, int, policy, const struct sched_param *, p);
+#define __NR___syscall_sched_setscheduler __NR_sched_setscheduler
+static inline
+_syscall3(int, __syscall_sched_setscheduler, __kernel_pid_t, pid, int, policy, const struct sched_param *, p);
+int sched_setscheduler(pid_t pid, int policy, const struct sched_param * p)
+{
+	return(__syscall_sched_setscheduler(pid, policy, p));
+}
 #endif
 #endif
 
 //#define __NR_sched_getscheduler       157
 #ifdef __NR_sched_getscheduler
-#ifdef L_sched_getscheduler
+#ifdef L___syscall_sched_getscheduler
 #include <sched.h>
-_syscall1(int, sched_getscheduler, pid_t, pid);
+#define __NR___syscall_sched_getscheduler __NR_sched_getscheduler
+static inline
+_syscall1(int, __syscall_sched_getscheduler, __kernel_pid_t, pid);
+int sched_getscheduler(pid_t pid)
+{
+	return(__syscall_sched_getscheduler(pid));
+}
 #endif
 #endif
 
@@ -1401,9 +1768,15 @@ _syscall1(int, sched_get_priority_min, int, policy);
 
 //#define __NR_sched_rr_get_interval    161
 #ifdef __NR_sched_rr_get_interval
-#ifdef L_sched_rr_get_interval
+#ifdef L___syscall_sched_rr_get_interval
 #include <sched.h>
-_syscall2(int, sched_rr_get_interval, pid_t, pid, struct timespec *, tp);
+#define __NR___syscall_sched_rr_get_interval __NR_sched_rr_get_interval
+static inline
+_syscall2(int, __syscall_sched_rr_get_interval, __kernel_pid_t, pid, struct timespec *, tp);
+int sched_rr_get_interval(pid_t pid, struct timespec * tp)
+{
+	return(__syscall_sched_rr_get_interval(pid, tp));
+}
 #endif
 #endif
 
@@ -1424,15 +1797,42 @@ _syscall4(__ptr_t, mremap, __ptr_t, old_address, size_t, old_size, size_t, new_s
 
 //#define __NR_setresuid                164
 #ifdef __NR_setresuid
-#ifdef L_setresuid
-_syscall3(int, setresuid, uid_t, ruid, uid_t, euid, uid_t, suid);
+#ifdef L___syscall_setresuid
+#define __NR___syscall_setresuid __NR_setresuid
+static inline
+_syscall3(int, __syscall_setresuid, __kernel_uid_t, rgid, __kernel_uid_t, egid, __kernel_uid_t, sgid);
+int setresuid(uid_t ruid, uid_t euid, uid_t suid)
+{
+	if (((ruid + 1) > (uid_t) ((__kernel_uid_t) -1U))
+			|| ((euid + 1) > (uid_t) ((__kernel_uid_t) -1U))
+			|| ((suid + 1) > (uid_t) ((__kernel_uid_t) -1U)))
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
+	return(__syscall_setresuid(ruid, euid, suid));
+}
 #endif
 #endif
 
 //#define __NR_getresuid                165
 #ifdef __NR_getresuid
-#ifdef L_getresuid
-_syscall3(int, getresuid, uid_t *, euid, uid_t *, ruid, uid_t *, suid);
+#ifdef L___syscall_getresuid
+#define __NR___syscall_getresuid __NR_getresuid
+static inline
+_syscall3(int, __syscall_getresuid, __kernel_uid_t *, ruid, __kernel_uid_t *, euid, __kernel_uid_t *, suid);
+int getresuid (uid_t *ruid, uid_t *euid, uid_t *suid)
+{
+	int result;
+	__kernel_uid_t k_ruid, k_euid, k_suid;
+	result = __syscall_getresuid(&k_ruid, &k_euid, &k_suid);
+	if (result == 0) {
+		*ruid = (uid_t) k_ruid;
+		*euid = (uid_t) k_euid;
+		*suid = (uid_t) k_suid;
+	}
+	return result;
+}
 #endif
 #endif
 
@@ -1454,9 +1854,14 @@ _syscall3(int, getresuid, uid_t *, euid, uid_t *, ruid, uid_t *, suid);
 #endif	
 
 //#define __NR_poll                     168
-#if defined(L_poll) && defined(__NR_poll) /* uClinux 2.0 doesn't have poll */
+#ifdef L_poll
+#ifdef __NR_poll
 #include <sys/poll.h>
 _syscall3(int, poll, struct pollfd *, fds, unsigned long int, nfds, int, timeout);
+#else
+/* uClinux 2.0 doesn't have poll, emulate it using select */
+#include "poll.c"
+#endif
 #endif
 
 //#define __NR_nfsservctl               169
@@ -1464,19 +1869,53 @@ _syscall3(int, poll, struct pollfd *, fds, unsigned long int, nfds, int, timeout
 
 //#define __NR_setresgid                170
 #ifdef __NR_setresgid
-#ifdef L_setresgid
-_syscall3(int, setresgid, gid_t, rgid, gid_t, egid, gid_t, sgid);
+#ifdef L___syscall_setresgid
+#define __NR___syscall_setresgid __NR_setresgid
+static inline
+_syscall3(int, __syscall_setresgid, __kernel_gid_t, rgid, __kernel_gid_t, egid, __kernel_gid_t, sgid);
+int setresgid(gid_t rgid, gid_t egid, gid_t sgid)
+{
+	if (((rgid + 1) > (gid_t) ((__kernel_gid_t) -1U))
+			|| ((egid + 1) > (gid_t) ((__kernel_gid_t) -1U))
+			|| ((sgid + 1) > (gid_t) ((__kernel_gid_t) -1U)))
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
+	return(__syscall_setresgid(rgid, egid, sgid));
+}
 #endif
 #endif
 
 //#define __NR_getresgid                171
 #ifdef __NR_getresgid
-#ifdef L_getresgid
-_syscall3(int, getresgid, gid_t *, egid, gid_t *, rgid, gid_t *, sgid);
+#ifdef L___syscall_getresgid
+#define __NR___syscall_getresgid __NR_getresgid
+static inline
+_syscall3(int, __syscall_getresgid, __kernel_gid_t *, egid, __kernel_gid_t *, rgid, __kernel_gid_t *, sgid);
+int getresgid(gid_t *rgid, gid_t *egid, gid_t *sgid)
+{
+	int result;
+	__kernel_gid_t k_rgid, k_egid, k_sgid;
+	result = __syscall_getresgid(&k_rgid, &k_egid, &k_sgid);
+	if (result == 0) { 
+		*rgid = (gid_t) k_rgid;
+		*egid = (gid_t) k_egid;
+		*sgid = (gid_t) k_sgid;
+	}
+	return result;
+}
 #endif
 #endif
 
 //#define __NR_prctl                    172
+#ifdef __NR_prctl
+#ifdef L_prctl
+#include <stdarg.h>
+//#include <sys/prctl.h>
+_syscall5(int, prctl, int, a, int, b, int, c, int, d, int, e);
+#endif
+#endif
 
 //#define __NR_rt_sigreturn             173
 //#define __NR_rt_sigaction             174
@@ -1585,9 +2024,21 @@ int sigsuspend (const sigset_t *mask)
 // See pread_write.c
 
 //#define __NR_chown                    182
-#ifdef L_chown
+#ifdef L___syscall_chown
 #include <unistd.h>
-_syscall3(int, chown, const char *, path, uid_t, owner, gid_t, group);
+#define __NR___syscall_chown __NR_chown
+static inline
+_syscall3(int, __syscall_chown, const char *, path, __kernel_uid_t, owner, __kernel_gid_t, group);
+int chown(const char * path, uid_t owner, gid_t group)
+{
+	if (((owner + 1) > (uid_t) ((__kernel_uid_t) -1U))
+			|| ((group + 1) > (gid_t) ((__kernel_gid_t) -1U)))
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
+	return(__syscall_chown(path, owner, group));
+}
 #endif
 
 //#define __NR_getcwd                   183
@@ -1670,7 +2121,7 @@ int getrlimit (__rlimit_resource_t resource, struct rlimit *rlimits)
 
 //#define __NR_stat64             195
 #ifdef L___syscall_stat64
-#if defined __NR_stat64 && defined __UCLIBC_HAS_LFS__
+#if defined __UCLIBC_HAS_LFS__ && defined __NR_stat64
 #define __NR___syscall_stat64 __NR_stat64
 #include <unistd.h>
 #include <sys/stat.h>
@@ -1692,7 +2143,7 @@ int stat64(const char * file_name, struct stat64 * buf)
 
 //#define __NR_lstat64            196
 #ifdef L___syscall_lstat64
-#if defined __NR_lstat64 && defined __UCLIBC_HAS_LFS__
+#if defined __UCLIBC_HAS_LFS__ && defined __NR_lstat64
 #define __NR___syscall_lstat64 __NR_lstat64
 #include <unistd.h>
 #include <sys/stat.h>
@@ -1714,7 +2165,7 @@ int lstat64(const char * file_name, struct stat64 * buf)
 
 //#define __NR_fstat64            197
 #ifdef L___syscall_fstat64
-#if defined __NR_fstat64 && defined __UCLIBC_HAS_LFS__
+#if defined __UCLIBC_HAS_LFS__ && defined __NR_fstat64
 #define __NR___syscall_fstat64 __NR_fstat64
 #include <unistd.h>
 #include <sys/stat.h>
@@ -1763,43 +2214,35 @@ _syscall2(int, pivot_root, const char *, new_root, const char *, put_old);
 
 //#define __NR_mincore		218
 //#define __NR_madvise		219
+#ifdef __NR_madvise
+#ifdef L_madvise
+_syscall3(int, madvise, void*, __addr, size_t, __len, int, __advice);
+#endif
+#endif
+
 //#define __NR_madvise1		219	/* delete when C lib stub is removed */
 
 //#define __NR_getdents64		220
-#ifdef L_getdents64
-#ifdef __UCLIBC_HAS_LFS__
-#include <unistd.h>
-#include <dirent.h>
-_syscall3(int, getdents64, int, fd, char *, dirp, size_t, count);
-#endif /* __UCLIBC_HAS_LFS__ */
-#endif
+// See getdents64.c
 
 //#define __NR_fcntl64		221
-#ifdef L__fcntl64
-#ifdef __UCLIBC_HAS_LFS__
-#ifdef __NR_fcntl64
-#define __NR__fcntl64 __NR_fcntl64
+#ifdef L___syscall_fcntl64
 #include <stdarg.h>
 #include <fcntl.h>
-extern int _fcntl64(int fd, int cmd, long arg);
-
-_syscall3(int, _fcntl64, int, fd, int, cmd, long, arg);
-
-int fcntl64(int fd, int command, ...)
+#if defined __UCLIBC_HAS_LFS__ && defined __NR_fcntl64
+#define __NR___syscall_fcntl64 __NR_fcntl64
+static inline
+_syscall3(int, __syscall_fcntl64, int, fd, int, cmd, long, arg);
+int __libc_fcntl64(int fd, int cmd, ...)
 {
 	long arg;
 	va_list list;
-
-	va_start(list, command);
+	va_start(list, cmd);
 	arg = va_arg(list, long);
-
 	va_end(list);
-	return _fcntl64(fd, command, arg);
+	return(__syscall_fcntl64(fd, cmd, arg));
 }
-#else
-extern int __libc_fcntl(int fd, int command, ...);
-weak_alias(__libc_fcntl, fcntl64)
-#endif
+weak_alias(__libc_fcntl64, fcntl64)
 #endif
 #endif
 
